/*****************************************************************************
*
*                         Techniques Class Library
*
*  ========================================================================
*
*   Copyright (C) 1991-2002 SciTech Software, Inc. All rights reserved.
*
*   This file may be distributed and/or modified under the terms of the
*   GNU Lesser General Public License version 2.1 as published by the Free
*   Software Foundation and appearing in the file LICENSE.LGPL included
*   in the packaging of this file.
*
*   Licensees holding a valid Commercial License for this product from
*   SciTech Software, Inc. may use this file in accordance with the
*   Commercial License Agreement provided with the Software.
*
*   This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
*   THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
*   PURPOSE.
*
*   See http://www.scitechsoft.com/license/ for information about
*   the licensing options available and how to purchase a Commercial
*   License Agreement.
*
*   Contact license@scitechsoft.com if any conditions of this licensing
*   are not clear to you, or you have questions about licensing options.
*
*  ========================================================================
*
* Language:     C++ 3.0
* Environment:  any
*
* Description:  Header file for command line parsing class. The class
*               TCOptions is an abstract class that defines the methods
*               used to parse the command line in standard System V
*               style.
*
****************************************************************************/

#ifndef __TCL_OPTIONS_HPP
#define __TCL_OPTIONS_HPP

#ifndef __SCITECH_H
#include "scitech.h"
#endif

#ifndef __IOSTREAM_HPP
#include <iostream.h>
#endif

/*---------------------------- Typedef's etc -----------------------------*/

#define OPT_ALLDONE     -1
#define OPT_PARAMETER   -2
#define OPT_INVALID     -3
#define OPT_HELP        -4

#define MAXARG      40

// TCOption type sepecifiers

#define OPT_INTEGER     'd'
#define OPT_HEX         'h'
#define OPT_OCTAL       'o'
#define OPT_UNSIGNED    'u'
#define OPT_LINTEGER    'D'
#define OPT_LHEX        'H'
#define OPT_LOCTAL      'O'
#define OPT_LUNSIGNED   'U'
#define OPT_FLOAT       'f'
#define OPT_DOUBLE      'F'
#define OPT_STRING      's'
#define OPT_SWITCH      '!'

/*--------------------------- Class Definition ----------------------------*/

//---------------------------------------------------------------------------
// The following structure is used to describe each of the options to the
// options parsing routines. An array of these will need to be set up
// in the constructor call for your derived TCOptions class below.
//---------------------------------------------------------------------------

struct TCOption {
    uchar   opt;                // The letter to describe the option
    uchar   type;               // Type descriptor for the option
    void    *arg;               // Place to store the argument
    char    *desc;              // Description for this option
    };

//---------------------------------------------------------------------------
// The following class is an abstract base class to provide the generalised
// methods to parse a command line. Descendants of this class will need
// to set up the optarr pointer in the constructor before parse() is
// called.
//---------------------------------------------------------------------------

class TCOptions {
protected:
    int         nextargv;       // Variable required by getopt
    char*       nextchar;
    TCOption    *optarr;        // Pointer to option desciption array
    int         numopt;         // Number of options in table

            // Protected member to parse a single option
            int getopt(int argc,char** argv,char* format,char** argument);

            // Protected member to convert a single option
            int convert(const TCOption& option,char* argument);

            // Protected pure virtual to parse a single parameter
    virtual int doParam(char* param,int num) = 0;

public:
            // Constructor
            TCOptions() { optarr = NULL; };

            // Method to parse the command line
            int parse(int argc,char *argv[]);

            // Friend function to display usage information
    friend  ostream& operator << (ostream& o,TCOptions& opt);
    };

#endif  // __TCL_OPTIONS_HPP
